import re

__version__ = '0.2'
__all__ = [
    'load', 'loads', 'Section'
]


def load(fp):
    """
    Deserialize the file as lynx format.
    Returns list of sections.
    """
    return loads(fp.read())

def loads(str):
    """
    Deserialize string as lynx format.
    Returns list of sections.
    """
    return Decoder().decode(str)


class Section(object):
    def __init__(self, name, sections, fields):
        self._name = name
        self._fields = fields
        self._sections = sections

    def name(self):
        """ Returns the section name """
        return self._name

    def sub_sections(self):
        """ Return a list of sub sections """
        return self._sections

    def fields(self):
        """ Return a dictionary of section's fields """
        return self._fields



class Decoder(object):
    def decode(self, str):
        self.str = str.strip()
        self.pos = 0
        result = []
        while self.pos < len(self.str):
            result.append(self._scan_part())
        return result

    def _scan_part(self):
        self._skip_comments()
        match = re.compile("[{}:]").search(self.str[self.pos:])
        name = self.str[self.pos:self.pos + match.start()].strip()
        c = match.group()
        self.pos += match.end()

        if (c == "{"):
            return self._scan_section(name)
        elif (c == "}"):
            return None
        elif (c == ":"):
            return self._scan_field(name)


    def _scan_section(self, name):
        fields = {}
        sections = []

        while True:
            result = self._scan_part()

            if result is None:
                break
            elif isinstance(result, Section):
                sections.append(result)
            else:
                fields.update(result)

        return Section(name, sections, fields)


    def _scan_field(self, name):
        match = re.compile("^\s*[|\[]").search(self.str[self.pos:])

        if match == None:
            return {name: self._scan_string()}

        self.pos += match.end()

        if match.group().strip() == "[":
            return {name: self._scan_list()}

    def _scan_string(self):
        match = re.compile("\n").search(self.str[self.pos:])
        value = self.str[self.pos:self.pos + match.start()].strip()
        self.pos += match.end()
        return value


    def _scan_list(self):
        match = re.compile("\]").search(self.str[self.pos:])
        value = self.str[self.pos:self.pos + match.start()].strip()
        self.pos += match.end()
        return [val.strip() for val in value.split(",")]

    def _skip_comments(self):
        match = re.compile("^\s*#").search(self.str[self.pos:])
        if not match is None:
            self.pos += match.end()
            match = re.compile("\n").search(self.str[self.pos:])
            self.pos += match.end()



from setuptools import setup, find_packages
import os


f = open(os.path.join(os.path.dirname(__file__), 'README.md'))
long_description = f.read().strip()
f.close()



setup(name='lynx',
      version='0.2.3',
      description = "Python configuration library",
      license="The MIT License (MIT)",
      url = "https://github.com/omershelef/lynx/",
      long_description = long_description,
      author = "Omer Shelef",
      author_email = "shlaflaf@gmail.com",
      packages=find_packages(
        where='.'
      ),
      tests_require=['pytest'],
      test_suite="tests.test_lynx.run_tests"
      )



import lynx
import sys
import pytest
import os

def run_tests():
    errno = pytest.main("tests/test_lynx.py")
    sys.exit(errno)


def load(filename):
    path = os.path.join(os.path.dirname(__file__), "config_files", filename)
    with open(path, "r") as fp:
        return lynx.load(fp)

def test_sections_works_correctly():
    config = load("sections.conf")
    assert(config[0].name() == config[1].name() == "section")
    sub_section2 = config[0].sub_sections()[0]
    assert(sub_section2.name() == "subsection2")
    assert(sub_section2.sub_sections()[0].name() == "subsection3")


def test_fields_works_correctly():
    config = load("fields.conf")
    fields = config[0].fields()
    assert(len(fields) == 2)
    assert(fields["my field"] == "hello world")
    assert(fields["hello~!&*#"] == "hello~!&*# world")


def test_lists_works_correctly():
    config = load("lists.conf")
    fields = config[0].fields()
    assert(len(fields) == 2)
    assert(fields["my list"] == ["1" , "myvalue", "3", "5"])
    assert(fields["list2"] == ["%$3", "!$^", "value2"])


